# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.security.publicca_v1beta1.types import resources

__protobuf__ = proto.module(
    package="google.cloud.security.publicca.v1beta1",
    manifest={
        "CreateExternalAccountKeyRequest",
    },
)


class CreateExternalAccountKeyRequest(proto.Message):
    r"""Creates a new
    [ExternalAccountKey][google.cloud.security.publicca.v1beta1.ExternalAccountKey]
    in a given project.

    Attributes:
        parent (str):
            Required. The parent resource where this
            external_account_key will be created. Format:
            projects/[project_id]/locations/[location]. At present only
            the "global" location is supported.
        external_account_key (google.cloud.security.publicca_v1beta1.types.ExternalAccountKey):
            Required. The external account key to create.
            This field only exists to future-proof the API.
            At present, all fields in ExternalAccountKey are
            output only and all values are ignored. For the
            purpose of the CreateExternalAccountKeyRequest,
            set it to a default/empty value.
    """

    parent = proto.Field(
        proto.STRING,
        number=1,
    )
    external_account_key = proto.Field(
        proto.MESSAGE,
        number=2,
        message=resources.ExternalAccountKey,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import Dict, Mapping, Optional, Sequence, Tuple, Type, Union

from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry as retries
from google.api_core.client_options import ClientOptions
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore
import pkg_resources

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object]  # type: ignore

from google.cloud.security.publicca_v1beta1.types import resources, service

from .client import PublicCertificateAuthorityServiceClient
from .transports.base import (
    DEFAULT_CLIENT_INFO,
    PublicCertificateAuthorityServiceTransport,
)
from .transports.grpc_asyncio import (
    PublicCertificateAuthorityServiceGrpcAsyncIOTransport,
)


class PublicCertificateAuthorityServiceAsyncClient:
    """Manages the resources required for ACME `external account
    binding <https://tools.ietf.org/html/rfc8555#section-7.3.4>`__ for
    the public certificate authority service.
    """

    _client: PublicCertificateAuthorityServiceClient

    DEFAULT_ENDPOINT = PublicCertificateAuthorityServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = (
        PublicCertificateAuthorityServiceClient.DEFAULT_MTLS_ENDPOINT
    )

    external_account_key_path = staticmethod(
        PublicCertificateAuthorityServiceClient.external_account_key_path
    )
    parse_external_account_key_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_external_account_key_path
    )
    common_billing_account_path = staticmethod(
        PublicCertificateAuthorityServiceClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(
        PublicCertificateAuthorityServiceClient.common_folder_path
    )
    parse_common_folder_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_common_folder_path
    )
    common_organization_path = staticmethod(
        PublicCertificateAuthorityServiceClient.common_organization_path
    )
    parse_common_organization_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_common_organization_path
    )
    common_project_path = staticmethod(
        PublicCertificateAuthorityServiceClient.common_project_path
    )
    parse_common_project_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_common_project_path
    )
    common_location_path = staticmethod(
        PublicCertificateAuthorityServiceClient.common_location_path
    )
    parse_common_location_path = staticmethod(
        PublicCertificateAuthorityServiceClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            PublicCertificateAuthorityServiceAsyncClient: The constructed client.
        """
        return PublicCertificateAuthorityServiceClient.from_service_account_info.__func__(PublicCertificateAuthorityServiceAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            PublicCertificateAuthorityServiceAsyncClient: The constructed client.
        """
        return PublicCertificateAuthorityServiceClient.from_service_account_file.__func__(PublicCertificateAuthorityServiceAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variabel is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return PublicCertificateAuthorityServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> PublicCertificateAuthorityServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            PublicCertificateAuthorityServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    get_transport_class = functools.partial(
        type(PublicCertificateAuthorityServiceClient).get_transport_class,
        type(PublicCertificateAuthorityServiceClient),
    )

    def __init__(
        self,
        *,
        credentials: ga_credentials.Credentials = None,
        transport: Union[
            str, PublicCertificateAuthorityServiceTransport
        ] = "grpc_asyncio",
        client_options: ClientOptions = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the public certificate authority service client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.PublicCertificateAuthorityServiceTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (ClientOptions): Custom options for the client. It
                won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = PublicCertificateAuthorityServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def create_external_account_key(
        self,
        request: Union[service.CreateExternalAccountKeyRequest, dict] = None,
        *,
        parent: str = None,
        external_account_key: resources.ExternalAccountKey = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> resources.ExternalAccountKey:
        r"""Creates a new
        [ExternalAccountKey][google.cloud.security.publicca.v1beta1.ExternalAccountKey]
        bound to the project.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud.security import publicca_v1beta1

            async def sample_create_external_account_key():
                # Create a client
                client = publicca_v1beta1.PublicCertificateAuthorityServiceAsyncClient()

                # Initialize request argument(s)
                request = publicca_v1beta1.CreateExternalAccountKeyRequest(
                    parent="parent_value",
                )

                # Make the request
                response = await client.create_external_account_key(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.cloud.security.publicca_v1beta1.types.CreateExternalAccountKeyRequest, dict]):
                The request object. Creates a new
                [ExternalAccountKey][google.cloud.security.publicca.v1beta1.ExternalAccountKey]
                in a given project.
            parent (:class:`str`):
                Required. The parent resource where this
                external_account_key will be created. Format:
                projects/[project_id]/locations/[location]. At present
                only the "global" location is supported.

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            external_account_key (:class:`google.cloud.security.publicca_v1beta1.types.ExternalAccountKey`):
                Required. The external account key to
                create. This field only exists to
                future-proof the API. At present, all
                fields in ExternalAccountKey are output
                only and all values are ignored. For the
                purpose of the
                CreateExternalAccountKeyRequest, set it
                to a default/empty value.

                This corresponds to the ``external_account_key`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.security.publicca_v1beta1.types.ExternalAccountKey:
                A representation of an ExternalAccountKey used for [external account
                   binding](\ https://tools.ietf.org/html/rfc8555#section-7.3.4)
                   within ACME.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([parent, external_account_key])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = service.CreateExternalAccountKeyRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent
        if external_account_key is not None:
            request.external_account_key = external_account_key

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.create_external_account_key,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


try:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
        gapic_version=pkg_resources.get_distribution(
            "google-cloud-public-ca",
        ).version,
    )
except pkg_resources.DistributionNotFound:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()


__all__ = ("PublicCertificateAuthorityServiceAsyncClient",)



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import dataclasses
import json  # type: ignore
import re
from typing import Callable, Dict, List, Optional, Sequence, Tuple, Union
import warnings

from google.api_core import gapic_v1, path_template, rest_helpers, rest_streaming
from google.api_core import exceptions as core_exceptions
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.auth.transport.requests import AuthorizedSession  # type: ignore
from google.protobuf import json_format
import grpc  # type: ignore
from requests import __version__ as requests_version

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object]  # type: ignore


from google.cloud.security.publicca_v1beta1.types import resources, service

from .base import DEFAULT_CLIENT_INFO as BASE_DEFAULT_CLIENT_INFO
from .base import PublicCertificateAuthorityServiceTransport

DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=BASE_DEFAULT_CLIENT_INFO.gapic_version,
    grpc_version=None,
    rest_version=requests_version,
)


class PublicCertificateAuthorityServiceRestInterceptor:
    """Interceptor for PublicCertificateAuthorityService.

    Interceptors are used to manipulate requests, request metadata, and responses
    in arbitrary ways.
    Example use cases include:
    * Logging
    * Verifying requests according to service or custom semantics
    * Stripping extraneous information from responses

    These use cases and more can be enabled by injecting an
    instance of a custom subclass when constructing the PublicCertificateAuthorityServiceRestTransport.

    .. code-block:: python
        class MyCustomPublicCertificateAuthorityServiceInterceptor(PublicCertificateAuthorityServiceRestInterceptor):
            def pre_create_external_account_key(request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_create_external_account_key(response):
                logging.log(f"Received response: {response}")

        transport = PublicCertificateAuthorityServiceRestTransport(interceptor=MyCustomPublicCertificateAuthorityServiceInterceptor())
        client = PublicCertificateAuthorityServiceClient(transport=transport)


    """

    def pre_create_external_account_key(
        self,
        request: service.CreateExternalAccountKeyRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[service.CreateExternalAccountKeyRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for create_external_account_key

        Override in a subclass to manipulate the request or metadata
        before they are sent to the PublicCertificateAuthorityService server.
        """
        return request, metadata

    def post_create_external_account_key(
        self, response: resources.ExternalAccountKey
    ) -> resources.ExternalAccountKey:
        """Post-rpc interceptor for create_external_account_key

        Override in a subclass to manipulate the response
        after it is returned by the PublicCertificateAuthorityService server but before
        it is returned to user code.
        """
        return response


@dataclasses.dataclass
class PublicCertificateAuthorityServiceRestStub:
    _session: AuthorizedSession
    _host: str
    _interceptor: PublicCertificateAuthorityServiceRestInterceptor


class PublicCertificateAuthorityServiceRestTransport(
    PublicCertificateAuthorityServiceTransport
):
    """REST backend transport for PublicCertificateAuthorityService.

    Manages the resources required for ACME `external account
    binding <https://tools.ietf.org/html/rfc8555#section-7.3.4>`__ for
    the public certificate authority service.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends JSON representations of protocol buffers over HTTP/1.1

    NOTE: This REST transport functionality is currently in a beta
    state (preview). We welcome your feedback via an issue in this
    library's source repository. Thank you!
    """

    def __init__(
        self,
        *,
        host: str = "publicca.googleapis.com",
        credentials: ga_credentials.Credentials = None,
        credentials_file: str = None,
        scopes: Sequence[str] = None,
        client_cert_source_for_mtls: Callable[[], Tuple[bytes, bytes]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        url_scheme: str = "https",
        interceptor: Optional[PublicCertificateAuthorityServiceRestInterceptor] = None,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        NOTE: This REST transport functionality is currently in a beta
        state (preview). We welcome your feedback via a GitHub issue in
        this library's repository. Thank you!

         Args:
             host (Optional[str]):
                  The hostname to connect to.
             credentials (Optional[google.auth.credentials.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify the application to the service; if none
                 are specified, the client will attempt to ascertain the
                 credentials from the environment.

             credentials_file (Optional[str]): A file with credentials that can
                 be loaded with :func:`google.auth.load_credentials_from_file`.
                 This argument is ignored if ``channel`` is provided.
             scopes (Optional(Sequence[str])): A list of scopes. This argument is
                 ignored if ``channel`` is provided.
             client_cert_source_for_mtls (Callable[[], Tuple[bytes, bytes]]): Client
                 certificate to configure mutual TLS HTTP channel. It is ignored
                 if ``channel`` is provided.
             quota_project_id (Optional[str]): An optional project to use for billing
                 and quota.
             client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                 The client info used to send a user-agent string along with
                 API requests. If ``None``, then default info will be used.
                 Generally, you only need to set this if you are developing
                 your own client library.
             always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                 be used for service account credentials.
             url_scheme: the protocol scheme for the API endpoint.  Normally
                 "https", but for testing or local servers,
                 "http" can be specified.
        """
        # Run the base constructor
        # TODO(yon-mg): resolve other ctor params i.e. scopes, quota, etc.
        # TODO: When custom host (api_endpoint) is set, `scopes` must *also* be set on the
        # credentials object
        maybe_url_match = re.match("^(?P<scheme>http(?:s)?://)?(?P<host>.*)$", host)
        if maybe_url_match is None:
            raise ValueError(
                f"Unexpected hostname structure: {host}"
            )  # pragma: NO COVER

        url_match_items = maybe_url_match.groupdict()

        host = f"{url_scheme}://{host}" if not url_match_items["scheme"] else host

        super().__init__(
            host=host,
            credentials=credentials,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            api_audience=api_audience,
        )
        self._session = AuthorizedSession(
            self._credentials, default_host=self.DEFAULT_HOST
        )
        if client_cert_source_for_mtls:
            self._session.configure_mtls_channel(client_cert_source_for_mtls)
        self._interceptor = (
            interceptor or PublicCertificateAuthorityServiceRestInterceptor()
        )
        self._prep_wrapped_messages(client_info)

    class _CreateExternalAccountKey(PublicCertificateAuthorityServiceRestStub):
        def __hash__(self):
            return hash("CreateExternalAccountKey")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, str] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: service.CreateExternalAccountKeyRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: float = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> resources.ExternalAccountKey:
            r"""Call the create external account
            key method over HTTP.

                Args:
                    request (~.service.CreateExternalAccountKeyRequest):
                        The request object. Creates a new
                    [ExternalAccountKey][google.cloud.security.publicca.v1beta1.ExternalAccountKey]
                    in a given project.

                    retry (google.api_core.retry.Retry): Designation of what errors, if any,
                        should be retried.
                    timeout (float): The timeout for this request.
                    metadata (Sequence[Tuple[str, str]]): Strings which should be
                        sent along with the request as metadata.

                Returns:
                    ~.resources.ExternalAccountKey:
                        A representation of an ExternalAccountKey used for
                    `external account
                    binding <https://tools.ietf.org/html/rfc8555#section-7.3.4>`__
                    within ACME.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v1beta1/{parent=projects/*/locations/*}/externalAccountKeys",
                    "body": "external_account_key",
                },
            ]
            request, metadata = self._interceptor.pre_create_external_account_key(
                request, metadata
            )
            pb_request = service.CreateExternalAccountKeyRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=False,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=False,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = resources.ExternalAccountKey()
            pb_resp = resources.ExternalAccountKey.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_create_external_account_key(resp)
            return resp

    @property
    def create_external_account_key(
        self,
    ) -> Callable[
        [service.CreateExternalAccountKeyRequest], resources.ExternalAccountKey
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._CreateExternalAccountKey(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def kind(self) -> str:
        return "rest"

    def close(self):
        self._session.close()


__all__ = ("PublicCertificateAuthorityServiceRestTransport",)



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.security.publicca.v1beta1",
    manifest={
        "ExternalAccountKey",
    },
)


class ExternalAccountKey(proto.Message):
    r"""A representation of an ExternalAccountKey used for `external account
    binding <https://tools.ietf.org/html/rfc8555#section-7.3.4>`__
    within ACME.

    Attributes:
        name (str):
            Output only. Resource name.
            projects/{project}/locations/{location}/externalAccountKeys/{key_id}
        key_id (str):
            Output only. Key ID.
            It is generated by the
            PublicCertificateAuthorityService when the
            ExternalAccountKey is created
        b64_mac_key (bytes):
            Output only. Base64-URL-encoded HS256 key.
            It is generated by the
            PublicCertificateAuthorityService when the
            ExternalAccountKey is created
    """

    name = proto.Field(
        proto.STRING,
        number=1,
    )
    key_id = proto.Field(
        proto.STRING,
        number=2,
    )
    b64_mac_key = proto.Field(
        proto.BYTES,
        number=3,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
